# A python-based UI module for iolite 4 starts with some metadata
#/ Type: UI
#/ Name: LiveCloud
#/ Description: Live plot of active selection
#/ Authors: J. Petrus
#/ References:
#/ Version: 1.0
#/ Contact:

from iolite.QtGui import QAction, QWidget, QVBoxLayout, QComboBox, QToolButton, QFrame, QCheckBox, QColorDialog
from iolite.QtCore import Qt, QSettings, QFile
from iolite.QtUiTools import QUiLoader
from iolite.ui import IolitePlotPyInterface as Plot

widget = None

class LiveCloudWidget(QWidget):

	def __init__(self, parent=None):
		QWidget.__init__(self, parent)
		self.setLayout(QVBoxLayout())
		self.layout().setContentsMargins(0, 0, 0, 0)
		settings = QSettings()
		uiPath = settings.value('Paths/UIPluginsPath')
		uiFile = QFile(uiPath + '/livecloud.ui')
		loader = QUiLoader()
		self.ui = loader.load(uiFile)
		self.layout().addWidget(self.ui)
		self.setWindowFlags(Qt.WindowStaysOnTopHint)
		self.setWindowTitle('Live Cloud')

		self.xComboBox = self.ui.findChild(QComboBox, 'xComboBox')
		self.yComboBox = self.ui.findChild(QComboBox, 'yComboBox')
		self.frame = self.ui.findChild(QFrame, 'frame')
		self.fixedCheckBox = self.ui.findChild(QCheckBox, 'fixedRangeCheckBox')
		self.showGroupCheckBox = self.ui.findChild(QCheckBox, 'showGroupCheckBox')
		self.selectionColorButton = self.ui.findChild(QToolButton, 'selectionColorButton')
		self.groupColorButton = self.ui.findChild(QToolButton, 'groupColorButton')

		self.initUi()

	def initUi(self):
		self.xComboBox.addItems(data.timeSeriesNames())
		self.yComboBox.addItems(data.timeSeriesNames())

		self.frame.setLayout(QVBoxLayout())
		self.plot = Plot(self.frame)
		self.plot.addLayer('group')
		self.plot.addLayer('selection')

		self.frame.layout().addWidget(self.plot)

		self.resize(700, 500)

		self.xComboBox.textActivated.connect(lambda t: self.updateSetting('xChannel', t))
		self.yComboBox.textActivated.connect(lambda t: self.updateSetting('yChannel', t))
		self.fixedCheckBox.toggled.connect(lambda b: self.updateSetting('fixed', b))
		self.showGroupCheckBox.toggled.connect(lambda b: self.updateSetting('showGroup', b))
		self.selectionColorButton.clicked.connect(self.getSelectionColor)
		self.groupColorButton.clicked.connect(self.getGroupColor)
		data.selectionChanged.connect(self.updatePlot)
		data.activeSelectionChanged.connect(self.updatePlot)

		self.xChannel = data.timeSeriesNames()[0]
		self.yChannel = data.timeSeriesNames()[0]
		self.showGroup = False
		self.selectionColor = Qt.blue
		self.groupColor = Qt.red
		self.fixed = False
		self.plottedGroup = None

	def updateSetting(self, name, value):
		setattr(self, name, value)
		self.plottedGroup = None
		self.updatePlot()

	def getSelectionColor(self):
		c = QColorDialog.getColor()
		self.updateSetting('selectionColor', c)

	def getGroupColor(self):
		c = QColorDialog.getColor()
		self.updateSetting('groupColor', c)

	def plotSelection(self, sel, color, layer):
		g = self.plot.addGraph()
		x = data.timeSeries(self.xChannel).dataForSelection(sel)
		y = data.timeSeries(self.yChannel).dataForSelection(sel)
		g.setData(x, y)
		g.setLineStyle('lsNone')
		g.setScatterStyle('ssDisc', 5, color, color)
		g.setLayer(layer)

	def updatePlot(self):
		group = data.activeSelectionGroup()
		
		try:
			updateGroup = group.name != self.plottedGroup
		except Exception as e:
			print(e)
			updateGroup = True

		if updateGroup or not self.showGroup:
			self.plot.clearLayer('group')

		if self.showGroup and updateGroup and group is not None:
			for gs in data.activeSelectionGroup().selections():
				self.plotSelection(gs, self.groupColor, 'group')
			self.plottedGroup = group.name
			self.plot.replot()

		self.plot.clearLayer('selection')
		activeSel = data.activeSelection()

		if activeSel:
			self.plotSelection(activeSel, self.selectionColor, 'selection')
		
		self.plot.left().setLabel(self.yChannel)
		self.plot.bottom().setLabel(self.xChannel)

		if not self.fixed:
			self.plot.rescaleAxes()

		self.plot.replot()



def createUIElements():
	action = QAction('Live Cloud', None)
	action.triggered.connect(createLiveCloud)
	ui.setAction(action)
	ui.setMenuName(['Tools'])

def createLiveCloud():
	global widget
	widget = LiveCloudWidget()
	widget.setAttribute(Qt.WA_DeleteOnClose)
	widget.show()
