# A python-based UI module for iolite 4 starts with some metadata
#/ Type: UI
#/ Name: RMExplorer
#/ Description: A new way to view RM Data
#/ Authors: J. Petrus
#/ References:
#/ Version: 1.0
#/ Contact:


from iolite.QtGui import QAction, QWidget, QVBoxLayout, QHBoxLayout, QGridLayout, QToolButton, QSplitter
from iolite.QtGui import QTableWidget, QTableWidgetItem, QBrush, QPen
from iolite.ui import CommonUIPyInterface as CUI, IolitePlotPyInterface as Plot, QCPBarsGroup, QCPColorGradient
from iolite.QtCore import Signal, Qt
import numpy as np
from functools import partial

class PeriodicTableWidget(QWidget):
	changed = Signal()

	cm = {
		's': (140, 198, 64),
		'p': (252, 176, 64),
		'd': (174, 223, 229),
		'f': (218, 150, 170)
	}

	def __init__(self, parent=None):
		QWidget.__init__(self, parent)
		self.setLayout(QGridLayout())
		self.setFixedSize(500, 250)
		self.layout().setContentsMargins(3, 3, 3, 3)
		self.elements = []

		for el in data.elements:
			b = QToolButton(self)
			b.setText(el)
			b.setCheckable(True)
			rgb = self.cm[data.elements[el]['block']]
			b.setStyleSheet('''
				QToolButton {
					background: rgb(%i, %i, %i);
					color: rgb(55, 55, 55);
				}
				QToolButton::checked {
					border: 2px solid rgb(175, 43, 59);
				}
				'''%(rgb[0], rgb[1], rgb[2])
			)
			r = data.elements[el]['period']
			c = data.elements[el]['group']
			if data.elements[el]['series'] > 8:
				r = data.elements[el]['series']
			self.layout().addWidget(b, r, c)
			b.toggled.connect(partial(self.updateElement, el))

	def updateElement(self, element, b):
		self.elements = [el for el in self.elements if el != element]

		if b:
			self.elements.append(element)

		self.changed.emit()

class RMExplorer(QWidget):

	def __init__(self, parent=None):
		QWidget.__init__(self, parent)
		self.setLayout(QHBoxLayout())
		self.pt = PeriodicTableWidget(self)
		self.layout().addWidget(self.pt)
		self.layout().setAlignment(self.pt, Qt.AlignTop)

		splitter = QSplitter(Qt.Vertical)
		self.rmTable = QTableWidget(self)
		self.plot = Plot(self)
		self.plot.setLegendVisible(True)
		self.plot.setLegendAlignment(Qt.AlignTop | Qt.AlignLeft)
		self.plot.bottom().setTickRotation(90)
		self.colorGrad = QCPColorGradient('gpSpectrum')
		self.barsGroup = QCPBarsGroup(self.plot)
		splitter.addWidget(self.rmTable)
		splitter.addWidget(self.plot)

		self.layout().addWidget(splitter)

		self.pt.changed.connect(self.updateRMs)
		self.pt.changed.connect(self.updatePlot)

	def updateRMs(self):
		self.rmTable.clear()

		if len(self.pt.elements) == 0:
			return

		matches = []
		for rmName in data.referenceMaterialNames():
			if all(el in data.referenceMaterialData(rmName) for el in self.pt.elements):
				matches.append(rmName)
		
		self.rmTable.setRowCount(len(matches))
		self.rmTable.setColumnCount(len(self.pt.elements) + 1)

		for mi, match in enumerate(matches):
			self.rmTable.setItem(mi, 0, QTableWidgetItem(match))

			for ei, el in enumerate(self.pt.elements):
				self.rmTable.setItem(mi, ei+1, QTableWidgetItem(str(data.referenceMaterialData(match)[el].valueInPPM())))

		self.rmTable.setHorizontalHeaderLabels(['Name'] + [f'{el} ppm' for el in self.pt.elements])
		self.rmNames = matches

	def updatePlot(self):
		self.plot.clearPlottables()

		x = np.arange(0, len(self.rmNames), dtype=np.float)

		for ei, el in enumerate(self.pt.elements):
			y = np.array([data.referenceMaterialData(rmName)[el].valueInPPM() for rmName in self.rmNames])
			b = self.plot.addBars(x, y)
			b.barsGroup = self.barsGroup
			b.width = 0.5/len(self.pt.elements)
			c = self.colorGrad.color(ei, 0, len(self.pt.elements))
			b.brush = QBrush(c)
			b.pen = QPen(c)
			b.name = el


		self.plot.left().setLabel('Concentration [ppm]')
		self.plot.bottom().setTicks(x, self.rmNames)

		self.plot.rescaleAxes()
		self.plot.replot()
			

def createUIElements():
	action = QAction('RM\nExplorer', None)
	action.setIcon(CUI().icon('balancescale'))
	ui.setAction(action)
	widget = RMExplorer()
	ui.setWidget(widget)

